package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesProdShiftRecordService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesProdShiftRecord;
import cn.estsh.i3plus.pojo.mes.repository.MesProdShiftRecordRepository;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;


/**
 * @Description: 开关班记录表
 * @Author: gsz
 * @Date: 2024/5/21 14:16
 * @Modify:
 */
@Service
@Slf4j
public class MesProdShiftRecordServiceImpl implements IMesProdShiftRecordService {

    @Autowired
    private MesProdShiftRecordRepository mesProdShiftRecordRDao;

    @Override
    public ListPager<MesProdShiftRecord> queryMesProdShiftRecordByPager(MesProdShiftRecord bean, Pager pager) {

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(bean.getWorkCenterCode(), "workCenterCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getShiftGroup(), "shiftGroup", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getShiftCode(), "shiftCode", packBean);
        if(!StringUtil.isEmpty(bean.orderBy())){
            packBean.setOrderByStr(bean.orderBy());
        }else {
            DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"createDatetime"}, packBean);
        }
        pager = PagerHelper.getPager(pager, mesProdShiftRecordRDao.findByHqlWhereCount(packBean));

        List<MesProdShiftRecord> resultList = mesProdShiftRecordRDao.findByHqlWherePage(packBean, pager);

        return new ListPager<>(resultList, pager);
    }

    @Override
    public void insertMesProdShiftRecord(MesProdShiftRecord item, String organizeCode) {
        //唯一性校验
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getWorkCenterCode(), "workCenterCode", packBean);
        DdlPreparedPack.getStringEqualPack(item.getShiftGroup(), "shiftGroup", packBean);
        DdlPreparedPack.getStringEqualPack(item.getShiftCode(), "shiftCode", packBean);

        boolean flg = mesProdShiftRecordRDao.isExitByHql(packBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("开关班记录产线【%s】班组代码【%s】班次代码【%s】已经存在，请检查数据",
                            item.getWorkCenterCode(),item.getShiftGroup(),item.getShiftCode())
                    .build();
        }
        ConvertBean.serviceModelInitialize(item, AuthUtil.getSessionUser().getUserName());
        mesProdShiftRecordRDao.save(item);
    }
    @Override
    public void saveMesProdShiftRecord(MesProdShiftRecord item, String organizeCode) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getWorkCenterCode(), "workCenterCode", packBean);
        DdlPreparedPack.getStringEqualPack(item.getShiftGroup(), "shiftGroup", packBean);
//        DdlPreparedPack.getStringEqualPack(item.getShiftCode(), "shiftCode", packBean);

        MesProdShiftRecord mesProdShiftRecord = mesProdShiftRecordRDao.getByProperty(packBean);
        if (StringUtil.isEmpty(mesProdShiftRecord)) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("开关班记录产线【%s】班组代码【%s】 不存在，请检查数据",
                            item.getWorkCenterCode(),item.getShiftGroup())
                    .build();
        }
        mesProdShiftRecord.setShiftGroup(item.getShiftGroup());
        mesProdShiftRecord.setShiftCode(item.getShiftCode());
        mesProdShiftRecord.setEndTime(item.getEndTime());
        ConvertBean.serviceModelUpdate(item, AuthUtil.getSessionUser().getUserName());
        mesProdShiftRecordRDao.save(item);
    }

    @Override
    public MesProdShiftRecord queryMesProdShiftRecord(MesProdShiftRecord bean) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(bean.getWorkCenterCode(), "workCenterCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getShiftGroup(), "shiftGroup", packBean);
        packBean.setOrderByStr(bean.orderBy());

        return mesProdShiftRecordRDao.getByProperty(packBean);
    }
}
