package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesAssemblyExtService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProdRuleCfgExtService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProdRuleContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesProdRuleNosortCfg;
import cn.estsh.i3plus.pojo.mes.repository.MesProdRuleNoSortCfgRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

@Slf4j
@Service
public class MesProdRuleCfgExtService implements IMesProdRuleCfgExtService {

    @Autowired
    private IMesAssemblyExtService assemblyExtService;

    @Autowired
    private MesProdRuleNoSortCfgRepository prodRuleNoSortCfgRepository;

    //【非排序线】获取产品加工规则
    @Override
    public List<MesProdRuleNosortCfg> getProdRuleNosortCfgList(MesProdRuleContext prodRuleContext) {
        if (StringUtils.isEmpty(prodRuleContext.getOrganizeCode()) || StringUtils.isEmpty(prodRuleContext.getEquipmentCode())) return null;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(prodRuleContext.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(prodRuleContext.getEquipmentCode(), MesPcnExtConstWords.EQUIPMENT_CODE, packBean);

        //outPartNo与outPartNos 带入条件必须互斥 , 此处查询时不作判断
        DdlPreparedPack.getStringEqualPack(prodRuleContext.getOutPartNo(), MesPcnExtConstWords.OUT_PART_NO, packBean);
        DdlPreparedPack.getInPackList(prodRuleContext.getOutPartNoList(), MesPcnExtConstWords.OUT_PART_NO, packBean);

        if (prodRuleContext.getInPartIsEmpty()) {
            //进料为NULL
            DdlPreparedPack.getIsNull(MesPcnExtConstWords.IN_PART_NO, packBean);
            DdlPreparedPack.getIsNull(MesPcnExtConstWords.IN_PART_NO_RULE, packBean);
        } else {
            //inPartNo与inPartNos 带入条件必须互斥 , 此处查询时不作判断
            DdlPreparedPack.getStringEqualPack(prodRuleContext.getInPartNo(), MesPcnExtConstWords.IN_PART_NO, packBean);
            DdlPreparedPack.getInPackList(prodRuleContext.getInPartNoList(), MesPcnExtConstWords.IN_PART_NO, packBean);
            //进料规则有值
            if (prodRuleContext.getInPartRuleIsNotEmpty()) DdlPreparedPack.getIsNotNull(MesPcnExtConstWords.IN_PART_NO_RULE, packBean);
        }

        return prodRuleNoSortCfgRepository.findByHqlWhere(packBean);

    }

    //【非排序线】获取产品加工规则
    @Override
    public List<MesProdRuleNosortCfg> getProdRuleNosortCfgList(MesProdRuleContext... prodRuleContexts) {
        if (null == prodRuleContexts) return null;
        List<MesProdRuleNosortCfg> prodRuleNosortCfgList = null;
        for (MesProdRuleContext prodRuleContext : prodRuleContexts) {
            if (null == prodRuleContext) continue;
            List<MesProdRuleNosortCfg> itemList = getProdRuleNosortCfgList(prodRuleContext);
            if (CollectionUtils.isEmpty(itemList)) continue;
            if (CollectionUtils.isEmpty(prodRuleNosortCfgList)) prodRuleNosortCfgList = new ArrayList<>();
            prodRuleNosortCfgList.addAll(itemList);
        }
        return prodRuleNosortCfgList;
    }

    //【非排序线】获取产品加工规则对应的装配件信息
    @Override
    public MesProdRuleContext getProdRuleContext(MesProdRuleContext prodRuleContext) {
        //获取非排序装配件配置信息
        if (CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue() == prodRuleContext.getIsHasBind()) prodRuleContext.assemblyDataJson(assemblyExtService.getProductionAssemblyNosortContextList(prodRuleContext));
        return prodRuleContext;
    }

    //【排序线】获取产品加工规则(条码对应的装配件绑定记录,目前条码等同工单)
    @Override
    public MesProdRuleContext getProdRuleSortContext(MesProdRuleContext prodRuleContext) {
        return prodRuleContext.assemblyDataJson(assemblyExtService.getProductionAssemblySortContextList(prodRuleContext));
    }


}
