package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesPartService;
import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesTemplateService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProduceSnPrintService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.util.MesPcnException;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesProduceSnPrintModel;
import cn.estsh.i3plus.mes.pcn.api.iservice.busi.ISyncFuncService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.codemaker.SnowflakeIdMaker;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.repository.MesPackingDefineDetailsRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesPackingDefineRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesProduceSnRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @Description : 条码补打
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/6/13 15:52
 * @Modify:
 **/
@Service
public class MesProduceSnPrintServiceImpl implements IMesProduceSnPrintService {

    @Autowired
    private MesPackingDefineDetailsRepository mesPackingDefineDetailsRDao;

    @Autowired
    private MesPackingDefineRepository mesPackingDefineRDao;

    @Autowired
    private MesProduceSnRepository mesProduceSnRDao;

    @Autowired
    private SnowflakeIdMaker snowflakeIdMaker;

    @Autowired
    private ISyncFuncService syncFuncService;

    @Autowired
    private IMesPartService mesPartService;

    @Autowired
    private IMesTemplateService mesTemplateService;

    @Override
    public MesProduceSnPrintModel findMesPackingDefine(MesProduceSnPrintModel mesProduceSnPrintModel) {
        //查询SAP下发物料信息
        MesPartSap mesPartSap = mesPartService.getMesPartSapByPartNo(mesProduceSnPrintModel.getPartNo(), mesProduceSnPrintModel.getOrganizeCode());
        mesProduceSnPrintModel.setQty(mesPartSap.getSnp());
        mesProduceSnPrintModel.setPartName(mesPartSap.getPartName());
        //查询包装定义明细
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(mesProduceSnPrintModel.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesProduceSnPrintModel.getPartNo(), "partNo", ddlPackBean);
        List<MesPackingDefineDetails> packingDefineDetails = mesPackingDefineDetailsRDao.findByHqlWhere(ddlPackBean);
        if (CollectionUtils.isEmpty(packingDefineDetails)) {
            MesPcnException.throwFlowException("物料【%s】未维护包装定义信息", mesProduceSnPrintModel.getPartNo());
        }
        //查询包装定义主表
        ddlPackBean = DdlPackBean.getDdlPackBean(mesProduceSnPrintModel.getOrganizeCode());
        DdlPreparedPack.getInPackList(packingDefineDetails.stream().map(MesPackingDefineDetails::getPackCode).collect(Collectors.toList()), "packCode", ddlPackBean);
        mesProduceSnPrintModel.setMesPackingDefineList(mesPackingDefineRDao.findByHqlWhere(ddlPackBean));
        return mesProduceSnPrintModel;
    }

    @Override
    public MesProduceSn findMesProduceSn(MesProduceSnPrintModel mesProduceSnPrintModel) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(mesProduceSnPrintModel.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesProduceSnPrintModel.getProductSn(), "productSn", ddlPackBean);
        MesProduceSn mesProduceSn = mesProduceSnRDao.getByProperty(ddlPackBean);
        if (Objects.isNull(mesProduceSn)) {
            MesPcnException.throwFlowException("条码【%s】信息不存在", mesProduceSnPrintModel.getProductSn());
        }
        return mesProduceSn;
    }

    @Override
    public MesProduceSnPrintModel doPrint(MesProduceSnPrintModel mesProduceSnPrintModel) {
        //物料信息
        MesPart mesPart = mesPartService.getMesPartByPartNo(mesProduceSnPrintModel.getPartNo(), mesProduceSnPrintModel.getOrganizeCode());
        //条码生成
        if (StringUtils.isEmpty(mesProduceSnPrintModel.getProductSn())) {
            ValidatorBean.checkNotNull(mesProduceSnPrintModel.getSnRule(), "条码规则不能为空");
            ValidatorBean.checkNotNull(mesProduceSnPrintModel.getPrintTemplate(), "打印模板不能为空");
            //生成条码
            List<String> snList = getStringList(mesProduceSnPrintModel);
            for (String sn : snList) {
                mesProduceSnPrintModel.getMesProduceSnList().add(generateMesProduceSn(mesPart, sn, mesProduceSnPrintModel.getUserName(),mesProduceSnPrintModel.getQty()));
            }
            mesProduceSnRDao.saveAll(mesProduceSnPrintModel.getMesProduceSnList());
            //条码补打
        } else {
            ValidatorBean.checkNotNull(mesPart.getProductLabelTemplate(), "打印模板不能为空");
            mesProduceSnPrintModel.setPrintTemplate(mesPart.getProductLabelTemplate());
            mesProduceSnPrintModel.getMesProduceSnList().add(findMesProduceSn(mesProduceSnPrintModel));
        }
        //获取打印模板信息
        mesProduceSnPrintModel.setMesLabelTemplate(mesTemplateService.getMesLabelTemplate(mesProduceSnPrintModel.getPrintTemplate()));
        return mesProduceSnPrintModel;
    }

    private List<String> getStringList(MesProduceSnPrintModel mesProduceSnPrintModel) {
        GenSerialNoModel genSerialNoModel = new GenSerialNoModel(mesProduceSnPrintModel.getSnRule());
        List<String> resultList = syncFuncService.syncSerialNo(genSerialNoModel, mesProduceSnPrintModel.getUserName(), mesProduceSnPrintModel.getOrganizeCode(), mesProduceSnPrintModel.getPrintQty()).getResultList();
        return resultList;
    }

    private MesProduceSn generateMesProduceSn(MesPart mesPart, String sn, String userName,Double qty) {
        MesProduceSn mesProduceSn = new MesProduceSn();
        mesProduceSn.setSerialNumber(snowflakeIdMaker.nextId() + "");
        mesProduceSn.setProductSn(sn);
        mesProduceSn.setCustSn(sn);
        mesProduceSn.setPartNo(mesPart.getPartNo());
        mesProduceSn.setPartName(mesPart.getPartName());
        mesProduceSn.setProcessLabelTemplate(mesPart.getProcessLabelTemplate());
        mesProduceSn.setCustLabelTemplate(mesPart.getCustLabelTemplate());
        mesProduceSn.setProdLabelTemplate(mesPart.getProductLabelTemplate());
        mesProduceSn.setQty(qty);
        mesProduceSn.setSnStatus(10);
        mesProduceSn.setQcStatus(MesExtEnumUtil.PRODUCE_QC_STATUS.QUALIFIED.getValue());
        mesProduceSn.setLotNo(TimeTool.getDateTimeShort());
        mesProduceSn.setPrintCount(1);
        mesProduceSn.setPrintStatus(20);
        mesProduceSn.setOrganizeCode(mesPart.getOrganizeCode());
        ConvertBean.serviceModelInitialize(mesProduceSn, userName);
        return mesProduceSn;
    }
}
