package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.station.function;

import cn.estsh.i3plus.mes.pcn.actor.shipping.dispatch.IFsmCommonService;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.BaseSwsService;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.IShippingDispatchService;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.function.IFsmModuleFunctionService;
import cn.estsh.i3plus.platform.common.util.MesPcnConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.MesPcnEnumUtil;
import cn.estsh.i3plus.pojo.mes.model.ButtonDynamicModel;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StationResultBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.Map;

/**
 * @Description : 工位参数按钮事件接口实现【重置工序】
 **/
@Service
public class MesFunctionJumpProcessService extends BaseSwsService implements IFsmModuleFunctionService {

    @Autowired
    private IFsmCommonService fsmCommonService;

    @Autowired
    private IShippingDispatchService shippingDispatchService;

    @Override
    public boolean doFunction(StationRequestBean reqBean, StationResultBean resultBean, ButtonDynamicModel buttonDynamicModel) {

        StationResultBean stationResultBean = new StationResultBean().busiType(buttonDynamicModel.getButtonCode()).dataType(MesPcnEnumUtil.STATION_DATA_TYPE.EXP_TEXT.getValue());

        Map<String, String> wcpcMap = fsmCommonService.doHandleFsmWcpcMapDataForDoScan(reqBean);
        String jumpState = wcpcMap.get(MesPcnEnumUtil.STATION_REQUEST_BEAN_CMD.JUMP_STATE.getCode());

        if (StringUtils.isEmpty(jumpState)) {
            this.sendMessage(reqBean, resultBean.writeDbLog(), String.format("生产线[%s]工位[%s]%s失败,未配置工位参数[%s]!",
                    reqBean.getWorkCenterCode(), reqBean.getWorkCellCode(), buttonDynamicModel.getButtonName(), MesPcnEnumUtil.STATION_REQUEST_BEAN_CMD.FORCE_JUMP_PROCESS.getCode()), MesPcnEnumUtil.STATION_BUSI_TYPE.MESSAGE, MesPcnEnumUtil.STATION_DATA_TYPE.EXP_TEXT);
            this.sendMessage(reqBean, stationResultBean.message(resultBean.getMessage()));
            return false;
        }

        this.sendMessage(reqBean, stationResultBean.dataType(MesPcnEnumUtil.STATION_DATA_TYPE.TEXT.getValue()));

        reqBean.setClientInfo(shippingDispatchService.getActorClientInfo(reqBean));
        reqBean.setInterfaceType(MesPcnConstWords.SHIPPING);
        reqBean.setBusiType(MesPcnConstWords.WS_CMD_DO_SCAN);
        reqBean.setForceJumpProcess(true);
        shippingDispatchService.doSendScanQueueNextExec(reqBean);

        return true;

    }

}
