package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.step;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionDispatchContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionPsOutContext;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.BaseStepService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.MathOperation;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StationResultBean;
import cn.estsh.i3plus.pojo.mes.model.StepResult;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description : 生产汇报工步 非排序报工
 *
 * @Author : zxw  --castle update
 * @Date: 2024/06/07
 **/
@Slf4j
@Service
public class MesReportNoSortStepService extends BaseStepService {

    @Autowired
    private IMesProductionDispatchContextStepService mesProductionDispatchContextStepService;

    @Autowired
    private MesPartRepository mesPartRDao;

    @Autowired
    private MesWorkOrderRepository workOrderRepository;

    @Autowired
    private MesWorkCenterRepository mesWorkCenterRDao;

    @Autowired
    private MesProductVersionRepository mesProductVersionRDao;

    @Autowired
    private MesBomRepository mesBomRDao;

    @Autowired
    private MesProductOffLineRepository mesProductOffLineRDao;

    @Autowired
    private MesProductPlanRepository mesProductPlanRDao;


    @Override
    public StepResult execute(StationRequestBean reqBean) {
        StationResultBean resultBean = new StationResultBean();
        StepResult stepResult = StepResult.getSuccessComplete();
        // todo 先不使用缓存中的数据
        //1. 校验当前有没有工单---只有有工单才能报工
        //如果产品加工规则中的foreignKey 和 产成零件信息 一一对应的,查询MesProductionPartContext的工单号
        List<MesProductionPsOutContext> mesProduceSns = mesProductionDispatchContextStepService.getProductionPsOutContext(reqBean);
        this.doProductReport(mesProduceSns, reqBean.getOrganizeCode(), reqBean.getUserInfo(),reqBean,resultBean);
        return execSuccessCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog(), stepResult, "报工成功");
  }

    public void doProductReport(List<MesProductionPsOutContext>  mesProduceSnList, String organizeCode, String userName, StationRequestBean reqBean, StationResultBean resultBean) {
        //新增初始化
        Map<String, List<MesProduceSn>> mesWorkOrderMap = mesProduceSnList.stream().collect(Collectors.groupingBy(MesProduceSn::getPartNo));
        Map<String, MesPart> mesPartMap = new HashMap<>();
        MesPart mesPart;
        //查询工单状态
        Integer[] orderStatus =new Integer[]{MesExtEnumUtil.ORDER_STATUS.RELEASE.getValue(),MesExtEnumUtil.ORDER_STATUS.PROCESS.getValue()};
        for (Map.Entry<String, List<MesProduceSn>> mesProduceSn : mesWorkOrderMap.entrySet()) {
            List<MesProduceSn> mesWorkOrderList = mesProduceSn.getValue();
            //获取物料信息
            if(mesPartMap.containsKey(mesProduceSn.getKey())){
                mesPart = mesPartMap.get(mesProduceSn.getKey());
            }else{
                //查询物料信息
                DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
                DdlPreparedPack.getStringEqualPack(mesProduceSn.getKey(), "partNo", ddlPackBean);
                mesPart = mesPartRDao.getByProperty(ddlPackBean);
                if(Objects.isNull(mesPart)){
                    execExpSendMsgAndThrowEx(reqBean, resultBean, String.format("物料【%s】信息不存在", mesProduceSn.getKey()));
                }
                mesPartMap.put(mesProduceSn.getKey(),mesPart);
            }
            //根据物料获取已发布的工单
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(mesPart.getPartNo(), "partNo", ddlPackBean);
            DdlPreparedPack.getInPackArray(orderStatus, "workOrderStatus", ddlPackBean);
            MesWorkOrder oldMesWorkOrder = workOrderRepository.getByProperty(ddlPackBean);
            if(Objects.isNull(oldMesWorkOrder)){
                execExpSendMsgAndThrowEx(reqBean, resultBean, String.format("物料【%s】状态未已发布的工单信息不存在",  mesPart.getPartNo()));
            }
            //查询工作中心
            ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(oldMesWorkOrder.getWorkCenterCode(), "workCenterCode", ddlPackBean);
            MesWorkCenter mesWorkCenter = mesWorkCenterRDao.getByProperty(ddlPackBean);
            if (Objects.isNull(mesWorkCenter)) {
                execExpSendMsgAndThrowEx(reqBean, resultBean, String.format("产线【%s】不存在",   oldMesWorkOrder.getWorkCenterCode()));
            }
            //获取生产版本
            ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(mesPart.getPartNo(), "partNo", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(mesWorkCenter.getErpWorkCenter(), "workCenterCode", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(oldMesWorkOrder.getProductVersion(), "productVersion", ddlPackBean);
            MesProductVersion mesProductVersion = mesProductVersionRDao.getByProperty(ddlPackBean);
            if (null == mesProductVersion) {
                execExpSendMsgAndThrowEx(reqBean, resultBean, String.format("物料【%s】生产版本信息不存在", mesPart.getPartNo()));
            }
            //物料+生产版本获取bom信息
            ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(mesPart.getPartNo(), "partNo", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(mesProductVersion.getAlternativePartList(), "bomVersion", ddlPackBean);
            List<MesBom> mesBoms = mesBomRDao.findByHqlWhere(ddlPackBean);
            if (CollectionUtils.isEmpty(mesBoms)) {
                execExpSendMsgAndThrowEx(reqBean, resultBean, String.format("物料【%s】生产版本【%s】对应bom信息不存在",  mesPart.getPartNo(), mesProductVersion.getProductVersion()));
            }
            oldMesWorkOrder.setNum(mesWorkOrderList.size());
            oldMesWorkOrder.setReportedQty(MathOperation.add(oldMesWorkOrder.getNum(), oldMesWorkOrder.getReportedQty()));
            //更新SAP计划完成数量
            saveMesProductPlan(oldMesWorkOrder, false, true, reqBean, resultBean);

            //更新工单状态
            double unCompleteQty = MathOperation.sub(oldMesWorkOrder.getQty(), oldMesWorkOrder.getReportedQty());
            oldMesWorkOrder.setUnCompleteQty(unCompleteQty > 0 ? unCompleteQty : 0);
            if (oldMesWorkOrder.getReportedQty() > oldMesWorkOrder.getQty()) {
                execExpSendMsgAndThrowEx(reqBean, resultBean, String.format("工单报工数量【%s】大于工单数量【%s】,不允许报工", oldMesWorkOrder.getReportedQty(), oldMesWorkOrder.getQty()));
            } else if (Objects.equals(oldMesWorkOrder.getReportedQty(), oldMesWorkOrder.getQty())) {
                oldMesWorkOrder.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.COMPLETE.getValue());
            } else {
                oldMesWorkOrder.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.PROCESS.getValue());
            }
            ConvertBean.serviceModelUpdate(oldMesWorkOrder,userName);
            workOrderRepository.update(oldMesWorkOrder);
            //保存数据
            List<MesProductOffLine> mesProductOffLineList = new ArrayList<>();
            MesProductOffLine newMesProductOffLine;
            for (MesProduceSn produceSn : mesWorkOrderList) {
                for (MesBom mesBom : mesBoms) {
                    newMesProductOffLine = new MesProductOffLine();
                    newMesProductOffLine.setReportPartNo(oldMesWorkOrder.getPartNo());
                    newMesProductOffLine.setReportPartNameRdd(oldMesWorkOrder.getPartName());
                    newMesProductOffLine.setItemPartNo(mesBom.getItemPartNo());
                    newMesProductOffLine.setItemPartName(mesBom.getItemPartName());
                    newMesProductOffLine.setItemQty(mesBom.getItemQty());
                    newMesProductOffLine.setAlort(mesProductVersion.getReceiveInventoryPoint());
                    newMesProductOffLine.setStgeLoc(mesProductVersion.getShipInventoryPoint());
                    newMesProductOffLine.setQty(1d);
                    newMesProductOffLine.setReportSn(produceSn.getProductSn());
                    newMesProductOffLine.setBomVersion(oldMesWorkOrder.getProductVersion());
                    newMesProductOffLine.setSerialNumber(produceSn.getProductSn());
                    newMesProductOffLine.setUnit(mesBom.getUnit());
                    newMesProductOffLine.setItemUnit(mesBom.getItemUnit());
                    newMesProductOffLine.setWorkOrderNo(oldMesWorkOrder.getWorkOrderNo());
                    newMesProductOffLine.setWorkCenterCode(oldMesWorkOrder.getWorkCenterCode());
                    newMesProductOffLine.setWorkCellCode(oldMesWorkOrder.getWorkCellCode());
                    newMesProductOffLine.setReportType(oldMesWorkOrder.getReportType());
                    newMesProductOffLine.setSapWorkCenter(oldMesWorkOrder.getErpWorkCenter());
                    newMesProductOffLine.setOrganizeCode(oldMesWorkOrder.getOrganizeCode());

                    ConvertBean.serviceModelInitialize(newMesProductOffLine, userName);
                    mesProductOffLineList.add(newMesProductOffLine);
                }
            }
            mesProductOffLineRDao.saveAll(mesProductOffLineList);
        }
    }

    private void saveMesProductPlan(MesWorkOrder bean, boolean isInsert, boolean isReport,StationRequestBean reqBean, StationResultBean resultBean) {
        if (StringUtil.isEmpty(bean.getPlanOrderNo())) {
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(bean.getPartNo(), "planPartNo", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(bean.getPlanStartTime(), "planStartDate", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(bean.getPlanEndTime(), "planEndDate", ddlPackBean);
            MesProductPlan mesProductPlan = mesProductPlanRDao.getByProperty(ddlPackBean);
            if (null == mesProductPlan) {
                MesProductPlan mesPlanOrder = new MesProductPlan();
                mesPlanOrder.setPlanOrderNo("");
                mesPlanOrder.setPlanQty(bean.getQty());
                mesPlanOrder.setPlanPartNo(bean.getPartNo());
                mesPlanOrder.setCompleteQty(0d);
                mesPlanOrder.setUncompleteQty(bean.getQty());
                mesPlanOrder.setUnit(bean.getUnit());
                mesPlanOrder.setPlanStartDate(bean.getPlanStartTime());
                mesPlanOrder.setPlanEndDate(bean.getPlanEndTime());
                mesPlanOrder.setPartMappingWorkCenterCode(bean.getErpWorkCenter());
                mesPlanOrder.setPlanOrganizeCode(bean.getOrganizeCode());
                mesPlanOrder.setOrganizeCode(bean.getOrganizeCode());
                ConvertBean.serviceModelInitialize(mesPlanOrder, bean.getCreateUser());
                mesProductPlanRDao.insert(mesPlanOrder);
            } else {
                if (isInsert) {
                    mesProductPlan.setPlanQty(MathOperation.add(mesProductPlan.getPlanQty(), bean.getQty()));
                } else {
                    //报工
                    if (isReport) {
                        mesProductPlan.setCompleteQty(MathOperation.add(mesProductPlan.getCompleteQty(), bean.getNum()));
                        //报工调整
                    } else {
                        mesProductPlan.setCompleteQty(MathOperation.sub(mesProductPlan.getCompleteQty(), bean.getNum()));
                    }
                }
                mesProductPlan.setUncompleteQty(MathOperation.sub(mesProductPlan.getPlanQty(), mesProductPlan.getCompleteQty()));
                ConvertBean.serviceModelUpdate(mesProductPlan, bean.getCreateUser());
                mesProductPlanRDao.update(mesProductPlan);
            }
        } else {
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(bean.getPlanOrderNo(), "planOrderNo", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(bean.getPartNo(), "planPartNo", ddlPackBean);
            MesProductPlan mesProductPlan = mesProductPlanRDao.getByProperty(ddlPackBean);
            if (null == mesProductPlan) {
                execExpSendMsgAndThrowEx(reqBean, resultBean, String.format("SAP计划单号【%s】物料【%s】,不存在", bean.getPlanOrderNo(), bean.getPartNo()));
            }
            if (!isInsert) {
                if (Objects.isNull(mesProductPlan.getCompleteQty())) {
                    mesProductPlan.setCompleteQty(0d);
                }
                //报工
                if (isReport) {
                    mesProductPlan.setCompleteQty(MathOperation.add(mesProductPlan.getCompleteQty(), bean.getNum()));
                    //报工调整
                } else {
                    mesProductPlan.setCompleteQty(MathOperation.sub(mesProductPlan.getCompleteQty(), bean.getNum()));
                }
                mesProductPlan.setUncompleteQty(MathOperation.sub(mesProductPlan.getPlanQty(), mesProductPlan.getCompleteQty()));
                ConvertBean.serviceModelUpdate(mesProductPlan, bean.getCreateUser());
                mesProductPlanRDao.update(mesProductPlan);
            }
        }
    }
}
