package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.step.context;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionCustomContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesCellEquipContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProdShiftContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.BaseStepService;
import cn.estsh.i3plus.mes.pcn.util.StationKvBeanUtil;
import cn.estsh.i3plus.pojo.base.enumutil.MesPcnEnumUtil;
import cn.estsh.i3plus.pojo.mes.model.StationKvBean;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StationResultBean;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import com.alibaba.fastjson.JSONObject;
import com.google.common.base.Objects;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.StringJoiner;

/**
 * @Description : 获取生产过程上下文对象接口实现【BUSI】
 * @Author : wangjie
 **/
@Slf4j
@Service("mesProductionCustomContextStepService")
public class MesProductionCustomContextStepService extends BaseStepService implements IMesProductionCustomContextStepService {

    //发送设备质量信息
    @Override
    public void sendEquipQualityMessage(StationRequestBean reqBean, MesCellEquipContext cellEquipContext) {
        StationResultBean resultBean = new StationResultBean()
                .busiType(MesPcnEnumUtil.STATION_BUSI_TYPE.QUALITY_MODULE.getValue()).dataType(MesPcnEnumUtil.STATION_DATA_TYPE.COLOR.getValue()).resultObj(MesExtEnumUtil.EQUIP_LOG_QUALITY.valueOfColor(cellEquipContext.getQuality()));
        this.sendMessage(reqBean, resultBean);
    }

    //工位加工锁KEY
    private String getProductionLockContextKey(StationRequestBean reqBean) { return getFsmBusikey(reqBean, new StringJoiner(MesPcnExtConstWords.COLON).add(MesPcnExtConstWords.PRODUCTION_PROCESS_CONTEXT).add(MesPcnExtConstWords.PRODUCTION_LOCK_CONTEXT).toString()); }


    //验证上下文工位加工锁是否存在
    @Override
    public Boolean checkProductionLockIsExistContext(StationRequestBean reqBean) {
        String value = getFsmBusiData(reqBean.getOrganizeCode(), getProductionLockContextKey(reqBean));
        return !StringUtils.isEmpty(value) ? true : false;
    }

    //保存上下文加工锁
    @Override
    public Boolean saveProductionLockContext(StationRequestBean reqBean, String value) {
        return saveFsmBusiData(reqBean.getOrganizeCode(), getProductionLockContextKey(reqBean), value);
    }

    //删除上下文加工锁
    @Override
    public void deleteProductionLockContext(StationRequestBean reqBean) {
        deleteFsmBusiData(reqBean.getOrganizeCode(), getProductionLockContextKey(reqBean));
    }

    //上个扫描/读取信息KEY
    private String getLastScanContextKey(StationRequestBean reqBean) { return getFsmBusikey(reqBean, new StringJoiner(MesPcnExtConstWords.COLON).add(MesPcnExtConstWords.PRODUCTION_PROCESS_CONTEXT).add(MesPcnExtConstWords.LAST_SCAN_INFO_CONTEXT).toString()); }

    //存储上个扫描/读取信息
    @Override
    public Boolean saveLastScanContext(StationRequestBean reqBean, List<StationKvBean> dataList) {
        return saveFsmBusiData(reqBean.getOrganizeCode(), getLastScanContextKey(reqBean), JSONObject.toJSONString(dataList));
    }

    //获取上个扫描/读取信息
    @Override
    public List<StationKvBean> getLastScanContext(StationRequestBean reqBean) {
        String lastScanContent = getFsmBusiData(reqBean.getOrganizeCode(), getLastScanContextKey(reqBean));
        return StringUtils.isEmpty(lastScanContent) ? new ArrayList<>() : JSONObject.parseArray(lastScanContent, StationKvBean.class);
    }

    //发送工步内容
    @Override
    public void doSendStepContextMessage(StationRequestBean reqBean) {
        sendStepContextMessage(reqBean, getLastScanContext(reqBean));
    }

    //发送工步内容
    @Override
    public void doSendStepContextMessage(StationRequestBean reqBean, String scanInfo, MesExtEnumUtil.CELL_MESSAGE_SOURCE source) {

        List<StationKvBean> lastScanContext = getLastScanContext(reqBean);

        lastScanContext = StationKvBeanUtil.addStationKvBeanLimix100ThenHalf(lastScanContext, new StationKvBean().name(source.getDescription()).value(scanInfo));

        saveLastScanContext(reqBean, lastScanContext);

        sendStepContextMessage(reqBean, lastScanContext);

    }

    //发送工步内容
    @Override
    public void sendStepContextMessage(StationRequestBean reqBean, List<StationKvBean> lastScanContext) {
        StationResultBean resultBean = new StationResultBean()
                .busiType(MesPcnEnumUtil.STATION_BUSI_TYPE.STEP_CONTENT.getValue()).dataType(MesPcnEnumUtil.STATION_DATA_TYPE.KEY_VALUE.getValue())
                .resultList(StationKvBeanUtil.addStationKvBeanList(new ArrayList<>(),
                        lastScanContext.size() > 1 ? lastScanContext.get(1).name("上次" + lastScanContext.get(1).getName()).key(MesPcnExtConstWords.LAST_SCAN_INFO) : null,
                        lastScanContext.size() > 0 ? lastScanContext.get(0).name("当前" + lastScanContext.get(0).getName()).key(MesPcnExtConstWords.SCAN_INFO) : null));
        this.sendMessage(reqBean, resultBean);
    }


    //手选腔数及工单信息KEY
    private String getChooseCavityOrderContextKey(StationRequestBean reqBean) { return getFsmBusikey(reqBean, new StringJoiner(MesPcnExtConstWords.COLON).add(MesPcnExtConstWords.PRODUCTION_PROCESS_CONTEXT).add(MesPcnExtConstWords.FUNCTION_CHOOSE_CAVITY_ORDER).toString()); }

    //获取手动选择的腔数及工单信息上下文
    @Override
    public List<StationKvBean> getFunctionChooseCavityOrderContext(StationRequestBean reqBean) {
        String chooseCavityOrderJson = getFsmBusiData(reqBean.getOrganizeCode(), getChooseCavityOrderContextKey(reqBean));
        return !StringUtils.isEmpty(chooseCavityOrderJson) ? JSONObject.parseArray(chooseCavityOrderJson, StationKvBean.class) : null;
    }

    //保存手动选择的腔数及工单信息上下文   腔数=工单个数  数据格式: [StationKvBean(key=cavity, name=腔数, value=3, seq=0), StationKvBean(key=workOrderNo, name=加工单, value=工单1;工单2;工单3, seq=1)]
    @Override
    public Boolean saveFunctionChooseCavityOrderContext(StationRequestBean reqBean, List<StationKvBean> resultList) {
        if (CollectionUtils.isEmpty(resultList)) return false;
        return saveFsmBusiData(reqBean.getOrganizeCode(), getChooseCavityOrderContextKey(reqBean), JSONObject.toJSONString(resultList));
    }

    //删除手动选择的腔数及工单信息上下文
    @Override
    public void deleteFunctionChooseCavityOrderContext(StationRequestBean reqBean) {
        deleteFsmBusiData(reqBean.getOrganizeCode(), getChooseCavityOrderContextKey(reqBean));
    }

    //开班信息KEY 【PS:生产线级别】
    private String getProdShiftContextKey(String orgainzeCode, String workCenterCode) { return new StringJoiner(MesPcnExtConstWords.COLON).add(orgainzeCode).add(MesPcnExtConstWords.PRODUCTION_PROCESS_CONTEXT).add(MesPcnExtConstWords.PROD_SHIFT_DATA_CONTEXT).add(workCenterCode).toString(); }




    //获取生产线的当前班组班次信息
    @Override
    public MesProdShiftContext getMesProdShiftKvBean(String orgainzeCode, String workCenterCode) {
        List<StationKvBean> stationKvBeans = getProdShiftDataContext(orgainzeCode, workCenterCode);

        MesProdShiftContext mesProdShiftContext = new MesProdShiftContext();
        if (CollectionUtils.isEmpty(stationKvBeans)) {
            return  null;
        }
        StationKvBean shiftGroupKvBean = stationKvBeans.stream().filter(stationKvBean -> Objects.equal(stationKvBean.getKey(), "shiftGroup")).findFirst().orElse(null);
        StationKvBean shiftCodeKvBean = stationKvBeans.stream().filter(stationKvBean -> Objects.equal(stationKvBean.getKey(), "shiftCode")).findFirst().orElse(null);

        mesProdShiftContext.setShiftCode(shiftGroupKvBean == null ? null : shiftGroupKvBean.getValue());
        mesProdShiftContext.setShiftGroup(shiftCodeKvBean == null ? null : shiftCodeKvBean.getValue());
        return  mesProdShiftContext;
    }

    //获取上下文班次班组信息
    @Override
    public List<StationKvBean> getProdShiftDataContext(String orgainzeCode, String workCenterCode) {
        String prodShiftJson = getFsmBusiData(orgainzeCode, getProdShiftContextKey(orgainzeCode, MesPcnExtConstWords.PROD_SHIFT_DATA_CONTEXT), workCenterCode);
        return !StringUtils.isEmpty(prodShiftJson) ? JSONObject.parseArray(prodShiftJson, StationKvBean.class) : null;
    }

    //验证上下文班次班组信息是否存在
    @Override
    public Boolean checkProdShiftDataIsExistContext(String orgainzeCode, String workCenterCode) {
        String prodShiftJson = getFsmBusiData(orgainzeCode, getProdShiftContextKey(orgainzeCode, MesPcnExtConstWords.PROD_SHIFT_DATA_CONTEXT), workCenterCode);
        return !StringUtils.isEmpty(prodShiftJson) ? true : false;
    }

    //保存上下文班次班组信息  [JSON]List<StationKvBean>
    @Override
    public Boolean saveProdShiftDataContext(String organizeCode, String workCenterCode, List<StationKvBean> prodShiftDataList) {
        if (CollectionUtils.isEmpty(prodShiftDataList)) return false;
        return saveFsmBusiData(organizeCode, getProdShiftContextKey(organizeCode, MesPcnExtConstWords.PROD_SHIFT_DATA_CONTEXT), workCenterCode, JSONObject.toJSONString(prodShiftDataList));
    }

    //删除上下文班次班组信息
    @Override
    public void deleteProdShiftDataContext(String orgainzeCode, String workCenterCode) {
        deleteFsmBusiData(orgainzeCode, getProdShiftContextKey(orgainzeCode, MesPcnExtConstWords.PROD_SHIFT_DATA_CONTEXT), workCenterCode);
    }

    //获取上下文工位加工数统计信息
    @Override
    public List<StationKvBean> getProductionStatisticsContext(StationRequestBean reqBean) {
        String productionStatisticsJson = getFsmBusiData(reqBean.getOrganizeCode(), getProductionLockContextKey(reqBean), MesPcnExtConstWords.PRODUCTION_STATISTICS_CONTEXT);
        return !StringUtils.isEmpty(productionStatisticsJson) ? JSONObject.parseArray(productionStatisticsJson, StationKvBean.class) : null;
    }

    //保存上下文工位加工数统计信息  [JSON]List<StationKvBean>
    @Override
    public Boolean saveProductionStatisticsContext(StationRequestBean reqBean, List<StationKvBean> productionStatisticsList) {
        if (CollectionUtils.isEmpty(productionStatisticsList)) return false;
        return saveFsmBusiData(reqBean.getOrganizeCode(), getProductionLockContextKey(reqBean), MesPcnExtConstWords.PRODUCTION_STATISTICS_CONTEXT, JSONObject.toJSONString(productionStatisticsList));
    }

    @Override
    public Boolean addProductionStatisticsContext(StationRequestBean reqBean, int count) {

        MesProdShiftContext mesProdShiftKvBean = getMesProdShiftKvBean(reqBean.getOrganizeCode(), reqBean.getWorkCenterCode());
        String key = new StringJoiner(MesPcnExtConstWords.AND).add(mesProdShiftKvBean.getShiftGroup()).add(mesProdShiftKvBean.getShiftCode()).toString();

        List<StationKvBean> stationKvBeans =  getProductionStatisticsContext(reqBean);
        if (CollectionUtils.isEmpty(stationKvBeans)) {
            String name = "加次数";
            String value = count + "";
            List<StationKvBean> generateStationKvBeans = StationKvBeanUtil.addStationKvBeanList(new ArrayList<>(), new StationKvBean(key, name, value),
                    new StationKvBean("color", "颜色", MesExtEnumUtil.COLOR.BLACK.getValue()));
            saveProductionStatisticsContext(reqBean, generateStationKvBeans);
        } else {
            stationKvBeans.stream().forEach(stationKvBean -> {
                if (Objects.equal(stationKvBean.getKey(), key)) {
                    stationKvBean.setValue(stationKvBean == null ? count + "" : (Integer.valueOf(stationKvBean.getValue()) + count) + "");
                }
            });
            saveProductionStatisticsContext(reqBean, stationKvBeans);
        }
        return  true;
    }

    //删除上下文工位加工数统计信息
    @Override
    public void deleteProductionStatisticsContext(StationRequestBean reqBean) {
        deleteFsmBusiData(reqBean.getOrganizeCode(), getProductionLockContextKey(reqBean), MesPcnExtConstWords.PRODUCTION_STATISTICS_CONTEXT);
    }

}
