package cn.estsh.i3plus.ext.mes.pcn.pojo.context;

import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesProduceSn;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkCell;
import io.swagger.annotations.ApiParam;
import lombok.Data;
import org.springframework.beans.BeanUtils;
import org.springframework.util.StringUtils;

import java.io.Serializable;

/**
 * 生产过程上下文对象-零件条码(非排序进料零件条码/排序工单号)
 */
@Data
public class MesProductionPsInContext implements Serializable {

    private static final long serialVersionUID = 4337949265011880288L;

    @ApiParam(name = "数据关联键  排序:生产工单号  非排序:snowflakeId")
    private Integer foreignKey;

    @ApiParam("组织代码")
    private String organizeCode;

    @ApiParam("零件条码ID")
    private Long id;

    @ApiParam("未知条码ID/队列ID")
    private Long relateId;

    @ApiParam("工单号")
    private String workOrderNo;

    @ApiParam("零件条码")
    private String productSn;

    @ApiParam("过程条码")
    private String serialNumber;

    @ApiParam("客户条码")
    private String custSn;

    @ApiParam("零件号")
    private String partNo;

    @ApiParam("零件名称")
    private String partName;

    @ApiParam("流程代码")
    private String routeCode;

    @ApiParam("工序代码")
    private String processCode;

    @ApiParam(value ="工艺代码")
    private String craftCode;

    @ApiParam("工艺跳过码")
    private String craftJumpCode;

    //默认否
    @ApiParam("是否空腔")
    private Integer isFinishCode = CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue();

    //默认否
    @ApiParam("是否顺序防错")
    private Integer isCheckSeq = CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue();

    //默认否
    @ApiParam(name = "是否前道工艺防错")
    private Integer isCheckCraft = CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue();

    //默认未验证
    @ApiParam("顺序防错结果")
    private Integer checkSeqResult = MesPcnExtConstWords.ZERO;

    //默认未验证
    @ApiParam(name = "前道工艺防错结果")
    private Integer checkCraftResult = MesPcnExtConstWords.ZERO;

    public MesProductionPsInContext() {}

    public MesProductionPsInContext foreignKey(Integer foreignKey) {
        this.foreignKey = foreignKey;
        return this;
    }

    public MesProductionPsInContext isCheck(MesWorkCell workCell) {
        this.isCheckSeq = workCell.getIsCheckSeq();
        this.isCheckCraft = workCell.getIsCheckCraft();
        return this;
    }

    public MesProductionPsInContext checkSeqResult(Integer checkSeqResult) {
        this.checkSeqResult = checkSeqResult;
        return this;
    }

    public MesProductionPsInContext checkCraftResult(Integer checkCraftResult) {
        this.checkCraftResult = checkCraftResult;
        return this;
    }

    public MesProductionPsInContext isFinishCode() {
        this.isFinishCode = CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue();
        return this;
    }

    public MesProductionPsInContext relateId(Long relateId) {
        this.relateId = relateId;
        return this;
    }

    public MesProductionPsInContext(String organizeCode, String productSn) {
        this.organizeCode = organizeCode;
        this.productSn = productSn;
    }

    public MesProductionPsInContext(MesWorkCell workCell, MesProduceSn produceSn) {
        BeanUtils.copyProperties(produceSn, this);
        this.isCheck(workCell);
    }

}
